"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ethers_1 = require("ethers");
var JSONUtils_1 = require("../utils/JSONUtils");
var utils_1 = require("../utils");
var UBAFeeUtility_1 = require("./UBAFeeUtility");
var UBAConfig = (function () {
    function UBAConfig(baselineFee, balancingFee, balanceTriggerThreshold, lpGammaFunction, incentivePoolAdjustment, ubaRewardMultiplier) {
        if (incentivePoolAdjustment === void 0) { incentivePoolAdjustment = {}; }
        if (ubaRewardMultiplier === void 0) { ubaRewardMultiplier = {}; }
        this.baselineFee = baselineFee;
        this.balancingFee = balancingFee;
        this.balanceTriggerThreshold = balanceTriggerThreshold;
        this.lpGammaFunction = lpGammaFunction;
        this.incentivePoolAdjustment = incentivePoolAdjustment;
        this.ubaRewardMultiplier = ubaRewardMultiplier;
        this.assertValidityOfAllFeeCurves();
    }
    UBAConfig.prototype.assertValidityOfAllFeeCurves = function () {
        var _a, _b;
        var omega = tslib_1.__spreadArray([this.balancingFee.default], Object.values((_a = this.balancingFee.override) !== null && _a !== void 0 ? _a : {}), true);
        var gamma = tslib_1.__spreadArray([this.lpGammaFunction.default], Object.values((_b = this.lpGammaFunction.override) !== null && _b !== void 0 ? _b : {}), true);
        omega.forEach(function (f) { return (0, UBAFeeUtility_1.assertValidityOfFeeCurve)(f, true); });
        gamma.forEach(function (f) { return (0, UBAFeeUtility_1.assertValidityOfFeeCurve)(f, false); });
    };
    UBAConfig.prototype.getBaselineFee = function (destinationChainId, originChainId) {
        var _a, _b, _c, _d, _e;
        return ((_e = (_d = (_b = (_a = this.baselineFee.override) === null || _a === void 0 ? void 0 : _a["".concat(originChainId, "-").concat(destinationChainId)]) !== null && _b !== void 0 ? _b : (_c = this.baselineFee.override) === null || _c === void 0 ? void 0 : _c["".concat(destinationChainId, "-").concat(originChainId)]) !== null && _d !== void 0 ? _d : this.baselineFee.default) !== null && _e !== void 0 ? _e : ethers_1.ethers.constants.Zero);
    };
    UBAConfig.prototype.getBalancingFeeTuples = function (chainId) {
        var _a, _b;
        return (_b = (_a = this.balancingFee.override) === null || _a === void 0 ? void 0 : _a[chainId]) !== null && _b !== void 0 ? _b : this.balancingFee.default;
    };
    UBAConfig.prototype.getZeroFeePointOnBalancingFeeCurve = function (chainId) {
        var balancingFeeTuples = this.getBalancingFeeTuples(chainId);
        var zeroPoint = balancingFeeTuples.find(function (tuple) { return tuple[1].eq(0); });
        if (!zeroPoint) {
            throw new Error("No zero point on balancing fee curve for chain ".concat(chainId));
        }
        return zeroPoint[0];
    };
    UBAConfig.prototype.isBalancingFeeCurveFlatAtZero = function (chainId) {
        var balancingFeeCurve = this.getBalancingFeeTuples(chainId);
        return balancingFeeCurve.length === 1 && balancingFeeCurve[0][0].eq(0) && balancingFeeCurve[0][1].eq(0);
    };
    UBAConfig.prototype.getLpGammaFunctionTuples = function (chainId) {
        var _a, _b;
        return (_b = (_a = this.lpGammaFunction.override) === null || _a === void 0 ? void 0 : _a[chainId]) !== null && _b !== void 0 ? _b : this.lpGammaFunction.default;
    };
    UBAConfig.prototype.getBalanceTriggerThreshold = function (chainId, tokenSymbol) {
        var _a, _b, _c;
        var chainTokenCombination = "".concat(chainId, "-").concat(tokenSymbol);
        return ((_c = (_b = (_a = this.balanceTriggerThreshold.override) === null || _a === void 0 ? void 0 : _a[chainTokenCombination]) !== null && _b !== void 0 ? _b : this.balanceTriggerThreshold.default) !== null && _c !== void 0 ? _c : {
            upperBound: {},
            lowerBound: {},
        });
    };
    UBAConfig.prototype.getTargetBalance = function (chainId, tokenSymbol) {
        var _a, _b;
        var thresholdConfig = this.getBalanceTriggerThreshold(chainId, tokenSymbol);
        return (_b = (_a = thresholdConfig === null || thresholdConfig === void 0 ? void 0 : thresholdConfig.upperBound) === null || _a === void 0 ? void 0 : _a.target) !== null && _b !== void 0 ? _b : ethers_1.ethers.constants.Zero;
    };
    UBAConfig.prototype.getIncentivePoolAdjustment = function (chainId) {
        var _a, _b;
        return (_b = (_a = this.incentivePoolAdjustment) === null || _a === void 0 ? void 0 : _a[chainId]) !== null && _b !== void 0 ? _b : ethers_1.ethers.constants.Zero;
    };
    UBAConfig.prototype.getUbaRewardMultiplier = function (chainId) {
        var _a, _b;
        return (_b = (_a = this.ubaRewardMultiplier) === null || _a === void 0 ? void 0 : _a[chainId]) !== null && _b !== void 0 ? _b : utils_1.fixedPointAdjustment;
    };
    UBAConfig.prototype.toJSON = function () {
        return {
            baselineFee: JSON.parse((0, JSONUtils_1.stringifyJSONWithNumericString)(this.baselineFee)),
            balancingFee: JSON.parse((0, JSONUtils_1.stringifyJSONWithNumericString)(this.balancingFee)),
            balanceTriggerThreshold: JSON.parse((0, JSONUtils_1.stringifyJSONWithNumericString)(this.balanceTriggerThreshold)),
            lpGammaFunction: JSON.parse((0, JSONUtils_1.stringifyJSONWithNumericString)(this.lpGammaFunction)),
            incentivePoolAdjustment: JSON.parse((0, JSONUtils_1.stringifyJSONWithNumericString)(this.incentivePoolAdjustment)),
            ubaRewardMultiplier: JSON.parse((0, JSONUtils_1.stringifyJSONWithNumericString)(this.ubaRewardMultiplier)),
        };
    };
    return UBAConfig;
}());
exports.default = UBAConfig;
//# sourceMappingURL=UBAFeeConfig.js.map