import { __spreadArray } from "tslib";
import { ethers } from "ethers";
import { stringifyJSONWithNumericString } from "../utils/JSONUtils";
import { fixedPointAdjustment } from "../utils";
import { assertValidityOfFeeCurve } from "./UBAFeeUtility";
/**
 * Defines the configuration needed to calculate the UBA fees
 */
var UBAConfig = /** @class */ (function () {
    /**
     * Instantiate a new UBA Config object
     * @param baselineFee A baseline fee that is applied to all transactions to allow LPs to earn a fee
     * @param balancingFee A record of piecewise functions for each chain and token that define the balancing fee to ensure either a positive or negative penalty to bridging a token to a chain that is either under or over utilized
     * @param balanceTriggerThreshold A record of boundry values for each chain and token that define the threshold for when the running balance should be balanced back to a fixed amount. Due to the fact that this type of operation is based on a heuristic and is considered a non-event transient property of the protocol, the threshold must be computed based on the current running balance of the spoke from the last validated running balance.
     * @param lpGammaFunction A record of piecewise functions for each chain that define the utilization fee to ensure that the bridge responds to periods of high utilization
     * @param incentivePoolAdjustment A DAO controlled variable to track any donations made to the incentivePool liquidity
     * @param ubaRewardMultiplier Used to scale rewards when a fee is larger than the incentive balance
     * @throws Error if any of the fee curves are invalid and assertValidityOfFeeCurves is true
     */
    function UBAConfig(baselineFee, balancingFee, balanceTriggerThreshold, lpGammaFunction, incentivePoolAdjustment, ubaRewardMultiplier) {
        if (incentivePoolAdjustment === void 0) { incentivePoolAdjustment = {}; }
        if (ubaRewardMultiplier === void 0) { ubaRewardMultiplier = {}; }
        this.baselineFee = baselineFee;
        this.balancingFee = balancingFee;
        this.balanceTriggerThreshold = balanceTriggerThreshold;
        this.lpGammaFunction = lpGammaFunction;
        this.incentivePoolAdjustment = incentivePoolAdjustment;
        this.ubaRewardMultiplier = ubaRewardMultiplier;
        // Validate the config
        this.assertValidityOfAllFeeCurves();
    }
    /**
     * Assert the validity of all fee curves. This is a helper function
     * that is called in the constructor to ensure that all fee curves
     * are valid.
     */
    UBAConfig.prototype.assertValidityOfAllFeeCurves = function () {
        var _a, _b;
        // Find all the fee curves that could possibiliy be used
        // in the UBA fee calculation. Specifically, these are the
        // balancing fee curve and the lp gamma function curve. The
        // curves are available for all overrides as well as their
        // default counterparts.
        var omega = __spreadArray([this.balancingFee.default], Object.values((_a = this.balancingFee.override) !== null && _a !== void 0 ? _a : {}), true);
        var gamma = __spreadArray([this.lpGammaFunction.default], Object.values((_b = this.lpGammaFunction.override) !== null && _b !== void 0 ? _b : {}), true);
        // Iterate through each curve and assert that it is valid
        omega.forEach(function (f) { return assertValidityOfFeeCurve(f, true); });
        gamma.forEach(function (f) { return assertValidityOfFeeCurve(f, false); });
    };
    /**
     * @description Get the baseline fee for a given route combination
     * @param destinationChainId The destination chain id
     * @param originChainId The origin chain id
     * @returns The baseline fee
     */
    UBAConfig.prototype.getBaselineFee = function (destinationChainId, originChainId) {
        var _a, _b, _c, _d, _e;
        return ((_e = (_d = (_b = (_a = this.baselineFee.override) === null || _a === void 0 ? void 0 : _a["".concat(originChainId, "-").concat(destinationChainId)]) !== null && _b !== void 0 ? _b : (_c = this.baselineFee.override) === null || _c === void 0 ? void 0 : _c["".concat(destinationChainId, "-").concat(originChainId)]) !== null && _d !== void 0 ? _d : this.baselineFee.default) !== null && _e !== void 0 ? _e : ethers.constants.Zero);
    };
    /**
     * @description Get the balancing fee tuples for a given chain
     * @param chainId The chain id
     * @returns The balancing fee
     */
    UBAConfig.prototype.getBalancingFeeTuples = function (chainId) {
        var _a, _b;
        return (_b = (_a = this.balancingFee.override) === null || _a === void 0 ? void 0 : _a[chainId]) !== null && _b !== void 0 ? _b : this.balancingFee.default;
    };
    UBAConfig.prototype.getZeroFeePointOnBalancingFeeCurve = function (chainId) {
        var balancingFeeTuples = this.getBalancingFeeTuples(chainId);
        var zeroPoint = balancingFeeTuples.find(function (tuple) { return tuple[1].eq(0); });
        if (!zeroPoint) {
            throw new Error("No zero point on balancing fee curve for chain ".concat(chainId));
        }
        return zeroPoint[0];
    };
    UBAConfig.prototype.isBalancingFeeCurveFlatAtZero = function (chainId) {
        var balancingFeeCurve = this.getBalancingFeeTuples(chainId);
        return balancingFeeCurve.length === 1 && balancingFeeCurve[0][0].eq(0) && balancingFeeCurve[0][1].eq(0);
    };
    /**
     * @description Get the lp gamma function tuples for a given chain
     * @param chainId The chain id
     * @returns The lp gamma function
     */
    UBAConfig.prototype.getLpGammaFunctionTuples = function (chainId) {
        var _a, _b;
        return (_b = (_a = this.lpGammaFunction.override) === null || _a === void 0 ? void 0 : _a[chainId]) !== null && _b !== void 0 ? _b : this.lpGammaFunction.default;
    };
    /**
     * @description Get the balance trigger threshold for a given chain and token
     * @param chainId The chain id
     * @param tokenSymbol The symbol of the token which will be resolved
     * @returns The balance trigger threshold if it exists
     */
    UBAConfig.prototype.getBalanceTriggerThreshold = function (chainId, tokenSymbol) {
        var _a, _b, _c;
        var chainTokenCombination = "".concat(chainId, "-").concat(tokenSymbol);
        return ((_c = (_b = (_a = this.balanceTriggerThreshold.override) === null || _a === void 0 ? void 0 : _a[chainTokenCombination]) !== null && _b !== void 0 ? _b : this.balanceTriggerThreshold.default) !== null && _c !== void 0 ? _c : {
            upperBound: {},
            lowerBound: {}, // Default to empty object if not set
        });
    };
    /**
     * Arbitrarily return upper bound target. This could be the average of the upper and lower bound targets but
     * for now return upper bound target.
     * @param chainId
     * @param l1TokenAddress
     * @returns
     */
    UBAConfig.prototype.getTargetBalance = function (chainId, tokenSymbol) {
        var _a, _b;
        var thresholdConfig = this.getBalanceTriggerThreshold(chainId, tokenSymbol);
        return (_b = (_a = thresholdConfig === null || thresholdConfig === void 0 ? void 0 : thresholdConfig.upperBound) === null || _a === void 0 ? void 0 : _a.target) !== null && _b !== void 0 ? _b : ethers.constants.Zero;
    };
    /**
     * Get the incentive pool adjustment
     * @param chainId The chain id
     * @returns The incentive pool adjustment. Defaults to 0 if not set
     */
    UBAConfig.prototype.getIncentivePoolAdjustment = function (chainId) {
        var _a, _b;
        return (_b = (_a = this.incentivePoolAdjustment) === null || _a === void 0 ? void 0 : _a[chainId]) !== null && _b !== void 0 ? _b : ethers.constants.Zero; // Default to 0 if not set
    };
    /**
     * Get the UBA reward multiplier
     * @param chainId The chain id
     * @returns The UBA reward multiplier. Defaults to 1 if not set
     */
    UBAConfig.prototype.getUbaRewardMultiplier = function (chainId) {
        var _a, _b;
        return (_b = (_a = this.ubaRewardMultiplier) === null || _a === void 0 ? void 0 : _a[chainId]) !== null && _b !== void 0 ? _b : fixedPointAdjustment; // Default to 1 if not set
    };
    UBAConfig.prototype.toJSON = function () {
        return {
            baselineFee: JSON.parse(stringifyJSONWithNumericString(this.baselineFee)),
            balancingFee: JSON.parse(stringifyJSONWithNumericString(this.balancingFee)),
            balanceTriggerThreshold: JSON.parse(stringifyJSONWithNumericString(this.balanceTriggerThreshold)),
            lpGammaFunction: JSON.parse(stringifyJSONWithNumericString(this.lpGammaFunction)),
            incentivePoolAdjustment: JSON.parse(stringifyJSONWithNumericString(this.incentivePoolAdjustment)),
            ubaRewardMultiplier: JSON.parse(stringifyJSONWithNumericString(this.ubaRewardMultiplier)),
        };
    };
    return UBAConfig;
}());
export default UBAConfig;
//# sourceMappingURL=UBAFeeConfig.js.map